@@ -18,13 +18,15 @@ import os
 import pathlib
 import pickle
 import platform
+import subprocess
 import sys
 from dataclasses import asdict
 from functools import lru_cache
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Any
 
 import cpuinfo
 import numpy as np
+import platformdirs
 
 import blosc2
 from blosc2 import blosc2_ext
@@ -36,6 +38,12 @@ if TYPE_CHECKING:
     import torch
 
 
+_USER_CACHE_DIR: pathlib.Path = platformdirs.user_cache_path(
+    appname="python-blosc2",
+    appauthor="blosc",
+)
+
+
 def _check_typesize(typesize):
     if not 1 <= typesize <= blosc2_ext.MAX_TYPESIZE:
         raise ValueError(f"typesize can only be in the 1-{blosc2_ext.MAX_TYPESIZE} range.")
@@ -113,6 +121,10 @@ def compress(
     instead of the python-blosc API variables like `blosc.SHUFFLE` for :paramref:`filter`
     or strings like "blosclz" for :paramref:`codec`.
 
+    This function only can deal with data < 2 GB.  If you want to compress
+    larger buffers, you should use the :class:`~blosc2.SChunk` class or, if you want to save
+    large arrays/tensors, the :func:`~blosc2.pack_tensor` function can be handier.
+
     Examples
     --------
     >>> import array, sys
@@ -121,6 +133,12 @@ def compress(
     >>> c_bytesobj = blosc2.compress(a_bytesobj, typesize=4)
     >>> len(c_bytesobj) < len(a_bytesobj)
     True
+
+    See also
+    --------
+    :func:`~blosc2.decompress`
+    :func:`~blosc2.pack_tensor`
+    :class:`~blosc2.SChunk`
     """
     len_src = len(src)
     if hasattr(src, "itemsize"):
@@ -626,7 +644,6 @@ def pack_tensor(
 
     Notes
     -----
-
     In case you pass a TensorFlow/PyTorch tensor, the tensor will be converted to a NumPy array
     before being packed. The tensor will be restored to its original form when unpacked.
 
@@ -1136,9 +1153,40 @@ def apple_silicon_cache_size(cache_level: int) -> int:
     return size.value
 
 
+def get_l3_cache_info():
+    result = subprocess.run(["lscpu", "--json"], capture_output=True, text=True)
+    lscpu_info = json.loads(result.stdout)
+    for entry in lscpu_info["lscpu"]:
+        if entry["field"] == "L3 cache:":
+            size_str, instances_str = entry["data"].split(" (")
+            size = int(size_str.split()[0]) * 1024 * 1024  # Convert MiB to bytes
+            instances = int(instances_str.split()[0])
+            return size, instances
+
+    raise ValueError("L3 cache not found in lscpu output")
+
+
 def linux_cache_size(cache_level: int, default_size: int) -> int:
     """Get the data cache_level size in bytes for Linux."""
     cache_size = default_size
+    if cache_level == 3:
+        # In modern multicore CPUs, the L3 cache is normally shared among all core complexes (CCX),
+        # but sysfs only reports the cache size for each complex, so better use lscpu, if available.
+        try:
+            l3_cache_size, l3_cache_instances = get_l3_cache_info()
+            # What comes next is a heuristic to guess the most appropriate L3 cache size.
+            # Essentially, this is the result of different experiments, mainly on AMD CPUs
+            # (in particular, Ryzen 9800X3D with 8 cores, and EPYC 9454P with 48 cores).
+            # For Intel, YMMV, but my guess is that they are not using the same CCX approach.
+            l3_cache_size *= l3_cache_instances
+            if l3_cache_instances > 1:
+                # This is yet another heuristic for large CPUs with core sets (CCX) having
+                # their own L3.  No idea why, but it seems to work well.
+                l3_cache_size *= l3_cache_instances // 2
+            return l3_cache_size
+        except (FileNotFoundError, ValueError):
+            # If lscpu is not available or the cache size cannot be read, try with sysfs
+            pass
     try:
         with open(f"/sys/devices/system/cpu/cpu0/cache/index{cache_level}/size") as f:
             size = f.read()
@@ -1162,7 +1210,8 @@ def _get_cpu_info():
     # cpuinfo does not correctly retrieve the cache sizes for all CPUs on Linux, so ask the kernel
     if platform.system() == "Linux":
         l1_data_cache_size = cpu_info.get("l1_data_cache_size", 32 * 1024)
-        cpu_info["l1_data_cache_size"] = linux_cache_size(1, l1_data_cache_size)
+        # Cache level 0 is typically the L1 data cache, and level 1 is the L1 instruction cache
+        cpu_info["l1_data_cache_size"] = linux_cache_size(0, l1_data_cache_size)
         l2_cache_size = cpu_info.get("l2_cache_size", 256 * 1024)
         cpu_info["l2_cache_size"] = linux_cache_size(2, l2_cache_size)
         l3_cache_size = cpu_info.get("l3_cache_size", 1024 * 1024)
@@ -1170,14 +1219,15 @@ def _get_cpu_info():
     return cpu_info
 
 
-def write_cached_cpu_info(cpu_info_dict: dict[str, any]) -> None:
-    with open(pathlib.Path.home() / ".blosc2-cpuinfo.json", "w") as f:
+def write_cached_cpu_info(cpu_info_dict: dict[str, Any]) -> None:
+    _USER_CACHE_DIR.mkdir(parents=True, exist_ok=True)
+    with (_USER_CACHE_DIR / "cpuinfo.json").open("w") as f:
         json.dump(cpu_info_dict, f, indent=4)
 
 
-def read_cached_cpu_info() -> dict:
+def read_cached_cpu_info() -> dict[str, Any]:
     try:
-        with open(pathlib.Path.home() / ".blosc2-cpuinfo.json") as f:
+        with (_USER_CACHE_DIR / "cpuinfo.json").open() as f:
             return json.load(f)
     except (FileNotFoundError, json.JSONDecodeError):
         return {}
@@ -1229,12 +1279,13 @@ def get_cbuffer_sizes(src: object) -> tuple[(int, int, int)]:
 
 
 # Compute a decent value for chunksize based on L3 and/or heuristics
-def get_chunksize(blocksize, l3_minimum=2**20, l3_maximum=2**26):
+def get_chunksize(blocksize, l3_minimum=4 * 2**20, l3_maximum=2**26):
     # Find a decent default when L3 cannot be detected by cpuinfo
     # Based mainly in heuristics
     chunksize = blocksize
     if blocksize * 32 < l3_maximum:
         chunksize = blocksize * 32
+
     # Refine with L2/L3 measurements (not always possible)
     cpu_info = blosc2.cpu_info
     if "l3_cache_size" in cpu_info:
@@ -1248,30 +1299,38 @@ def get_chunksize(blocksize, l3_minimum=2**20, l3_maximum=2**26):
             l2_cache_size = cpu_info.get("l2_cache_size", "Not found")
             if isinstance(l2_cache_size, int) and l3_cache_size > l2_cache_size:
                 chunksize = l3_cache_size
+        # When computing expressions, it is convenient to keep chunks for all operands
+        # in L3 cache, so let's divide by 4 (3 operands + result is a typical situation
+        # for moderately complex expressions)
+        chunksize //= 4
+
     # Chunksize should be at least the size of L2
     l2_cache_size = cpu_info.get("l2_cache_size", "Not found")
     if isinstance(l2_cache_size, int) and l2_cache_size > chunksize:
-        chunksize = l2_cache_size
-
-    # When evaluating expressions, it is convenient to keep chunks for all operands in L3 cache,
-    # so let's divide by 4 (3 operands + result is a typical situation for moderately complex
-    # expressions)
-    chunksize //= 4
+        # Apple Silicon has a large L2 cache, and memory bandwidth is high,
+        # so we can use a larger chunksize based on L2 cache size
+        chunksize = l2_cache_size * 4
 
     # Ensure a minimum size
     if chunksize < l3_minimum:
         chunksize = l3_minimum
 
-    # In Blosc2, the chunksize cannot be larger than 2 GB - BLOSC2_MAX_BUFFERSIZE
-    if chunksize > 2**31 - blosc2.MAX_OVERHEAD:
-        chunksize = 2**31 - blosc2.MAX_OVERHEAD
+    # In Blosc2, the chunksize cannot be larger than MAX_BUFFERSIZE
+    if chunksize > blosc2.MAX_BUFFERSIZE:
+        chunksize = blosc2.MAX_BUFFERSIZE
+
+    # chunksize can never be larger than blocksize
+    if chunksize < blocksize:
+        chunksize = blocksize
 
     return chunksize
 
 
-def nearest_divisor(a, b):
-    if a > 100_000:
-        # When `a` is largish, use a faster algorithm that only goes downwards
+def nearest_divisor(a, b, strict=False):
+    if a > 100_000 or strict:
+        # When `a` is largish, or we require `b` strictly less than `a`,
+        # use a (faster) algorithm that only goes downwards.
+        # This is quite brute force, and tried to optimize this, but I have not found a faster way.
         for i in range(b, 0, -1):
             if a % i == 0:
                 return i
@@ -1283,6 +1342,15 @@ def nearest_divisor(a, b):
     return min(divisors, key=lambda x: abs(x - b))
 
 
+# This could be a good alternative to nearest_divisor that deserves more testing
+# Found at: https://gist.github.com/raphaelvallat/5d5af7205df720db53be4cc2ee7e7549
+def find_closest_divisor(n, m):
+    """Find the divisor of n closest to m"""
+    divisors = np.array([i for i in range(1, int(np.sqrt(n) + 1)) if n % i == 0])
+    divisions = n // divisors
+    return divisions[np.argmin(np.abs(m - divisions))]
+
+
 # Compute chunks and blocks partitions
 def compute_partition(nitems, maxshape, minpart=None):
     if 0 in maxshape:
@@ -1304,7 +1372,7 @@ def compute_partition(nitems, maxshape, minpart=None):
             partition[-(i + 1)] = rsize
         else:
             rsize = max(max_items, minsize)
-            new_rsize = rsize if size % rsize == 0 else nearest_divisor(size, rsize)
+            new_rsize = rsize if size % rsize == 0 else nearest_divisor(size, rsize, strict=True)
             # If the new rsize is not too far from the original rsize, use it
             if rsize // 2 < new_rsize < rsize * 2:
                 rsize = new_rsize
@@ -1315,7 +1383,7 @@ def compute_partition(nitems, maxshape, minpart=None):
 
 
 def compute_chunks_blocks(  # noqa: C901
-    shape: tuple[int] | list,
+    shape: tuple | list,
     chunks: tuple | list | None = None,
     blocks: tuple | list | None = None,
     dtype: np.dtype = np.uint8,
@@ -1376,6 +1444,8 @@ def compute_chunks_blocks(  # noqa: C901
         return chunks, blocks
 
     cparams = kwargs.get("cparams") or copy.deepcopy(blosc2.cparams_dflts)
+    if isinstance(cparams, blosc2.CParams):
+        cparams = asdict(cparams)
     # Typesize in dtype always has preference over typesize in cparams
     itemsize = cparams["typesize"] = np.dtype(dtype).itemsize
 
@@ -1399,26 +1469,42 @@ def compute_chunks_blocks(  # noqa: C901
         cparams2["tuner"] = blosc2.Tuner.STUNE
         src = blosc2.compress2(np.zeros(nitems, dtype=f"V{itemsize}"), **cparams2)
         _, _, blocksize = blosc2.get_cbuffer_sizes(src)
-        # Maximum blocksize calculation
-        max_blocksize = blocksize
+        # Minimum blocksize calculation
+        min_blocksize = blocksize
         if platform.machine() == "x86_64":
-            # For modern Intel/AMD archs, experiments say to use half of the L2 cache size
-            max_blocksize = blosc2.cpu_info["l2_cache_size"] // 2
+            # For modern Intel/AMD archs, experiments say to split the cache among the operands
+            min_blocksize = blosc2.cpu_info["l2_cache_size"] // 4
+            if blosc2.cpu_info["l2_cache_size"] >= 2**21:
+                # Incidentally, some modern Intel CPUs have a larger L2 cache (2 MB) and they
+                # prefer smaller blocks.  This is somewhat heuristic, but it seems to work well.
+                min_blocksize = blosc2.cpu_info["l1_data_cache_size"] * 4
+            # New experiments say that using the 4x of the L1 size is even better
+            # But let's avoid this because it does not work well for AMD archs
+            # min_blocksize = blosc2.cpu_info["l1_data_cache_size"] * 4
         elif platform.system() == "Darwin" and "arm" in platform.machine():
-            # For Apple Silicon, experiments say to use half of the L1 cache size
-            max_blocksize = blosc2.cpu_info["l1_data_cache_size"] // 2
-        if "clevel" in cparams and cparams["clevel"] == 0:
-            # Experiments show that, when no compression is used, it is not a good idea
-            # to exceed half of private cache for the blocksize because speed suffers
-            # too much during evaluations.
-            blocksize = max_blocksize
-        elif blocksize > max_blocksize:
-            blocksize = max_blocksize
+            # For Apple Silicon, experiments say we can use 4x the L1 size
+            min_blocksize = blosc2.cpu_info["l1_data_cache_size"] * 2
+        elif "l1_data_cache_size" in blosc2.cpu_info and isinstance(
+            blosc2.cpu_info["l1_data_cache_size"], int
+        ):
+            # For other archs, we don't have hints; be conservative and use 2x the L1 size
+            min_blocksize = blosc2.cpu_info["l1_data_cache_size"] * 2
+
+        if blocksize < min_blocksize:
+            blocksize = min_blocksize
+
+        # Fix for #364
+        if blocksize < itemsize:
+            blocksize = itemsize
 
         cparams2["tuner"] = aux_tuner
     else:
         blocksize = math.prod(blocks) * itemsize
 
+    # Check limits for blocksize
+    if blocksize > blosc2.MAX_BLOCKSIZE:
+        raise ValueError("blocksize is too large: it cannot exceed MAX_BLOCKSIZE (~512MB)")
+
     # Now that a sensible blocksize has been computed, let's compute the blocks
     if chunks is None:
         maxshape = shape
@@ -1466,6 +1552,27 @@ def compress2(src: object, **kwargs: dict) -> str | bytes:
         If the data cannot be compressed into `dst`.
         If an internal error occurs, likely due to an
         invalid parameter.
+
+    Notes
+    -----
+    This function only can deal with data < 2 GB.  If you want to compress
+    larger buffers, you should use the :class:`~blosc2.SChunk` class or, if you want to save
+    large arrays/tensors, the :func:`~blosc2.pack_tensor` function can be handier.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> data = np.arange(1e6, dtype=np.float32)
+    >>> cparams = blosc2.CParams()
+    >>> compressed_data = blosc2.compress2(data, cparams=cparams)
+    >>> print(f"Compressed data length: {len(compressed_data)} bytes")
+    Compressed data length: 14129 bytes
+
+    See also
+    --------
+    :func:`~blosc2.decompress2`
+    :func:`~blosc2.pack_tensor`
+    :class:`~blosc2.SChunk`
     """
     if kwargs is not None and "cparams" in kwargs:
         if len(kwargs) > 1:
